#ifdef MAGIC_SELF
 
magic_int    (width,    0,    G_MAXINT,  0)
magic_int    (height,   0,    G_MAXINT,  0)

#else

#define MAGIC_SELF                 "op_scale.c"
#define MAGIC_SUPER_CLASS_FILTER
#define MAGIC_TypeName             OpScale
#define MAGIC_TypeNameClass        OpScaleClass
#define MAGIC_type_name            op_scale
#define MAGIC_type_name_init       op_scale_init
#define MAGIC_type_name_class_init op_scale_class_init
#include "magic.h"

static void
op_scale (Canvas *canvas,
          gint    dst_x,
          gint    dst_y,
          gint    dst_width,
          gint    dst_height,
          gint    src_x,
          gint    src_y,
          gint    src_width,
          gint    src_height);

#include <stdio.h>

/* Actual image processing code
 ************************************************************************/
static gboolean
evaluate (GeglOperation *operation,
          const gchar   *output_prop)
{
  OpFilter *filter = OP_FILTER(operation);
  OpScale  *self   = (OpScale*)(filter);

  CanvasRect *input  = filter->input;
  CanvasRect *output;

  if(strcmp("output", output_prop))
    return FALSE;

  if (filter->in_place)
    {
       g_warning ("not able to do inplace");
       return FALSE;
    }
  else
    {
      output = CANVAS_RECT (canvas_alloc (input->canvas, self->width, self->height));

      op_scale (output->canvas,
                input->x, input->y, input->width, input->height,
                output->x, output->y, output->width, output->height);
      if (filter->output)
        g_object_unref (filter->output);
      filter->output = output;
    }
  return  TRUE;
}

static void
op_scale (Canvas *canvas,
          gint    dst_x,
          gint    dst_y,
          gint    dst_width,
          gint    dst_height,
          gint    src_x,
          gint    src_y,
          gint    src_width,
          gint    src_height)
{
  gint x,y;
  guchar *src_buf;
  guchar *dst_buf;
  CanvasRect *src_rect;
  CanvasRect *dst_rect;

  src_buf = g_malloc0 (src_width * src_height * 4);
  if (!src_buf)
    return;
  dst_buf = g_malloc0 (dst_width * dst_height * 4);
  if (!dst_buf)
    {
      g_free (src_buf);
      return;
    }

  dst_rect = canvas_rect_new (canvas, dst_x, dst_y, dst_width, dst_height);
  canvas_rect_export_buf (dst_rect, dst_buf);
  g_object_unref (dst_rect);
  
  for (y=0;y<src_height;y++)
    for (x=0;x<src_width;x++)
      {
        gint i;
        for (i=0;i<4;i++)
          {
            gint u,v;
            u = x * dst_width / src_width;
            v = y * dst_height / src_height;
            src_buf [(y * src_width + x) * 4 + i] = 
              dst_buf [(v * dst_width + u) * 4 + i];
          }
      }
  g_free (dst_buf);
  
  src_rect = canvas_rect_new (canvas, src_x, src_y, src_width, src_height);
  canvas_rect_import_buf (src_rect, src_buf);
  g_object_unref (src_rect);

  g_free (src_buf);
}

#endif
