#include <string.h>

#include <glib-object.h>

#include "gegl/gegl-types.h"
#include "gegl/gegl-pad.h"

#include "op_sink.h"
#include "canvas/types.h"
#include "canvas/canvas_rect.h"
#include "canvas/canvas.h"

enum
{
  PROP_0,
  PROP_INPUT,
  PROP_PROCESS,
  PROP_LAST
};

static void  op_sink_class_init (OpSinkClass *klass);
static void  op_sink_init       (OpSink      *self);

static void     finalize     (GObject      *gobject);
static void     get_property (GObject      *gobject,
                              guint         prop_id,
                              GValue       *value,
                              GParamSpec   *pspec);
static void     set_property (GObject      *gobject,
                              guint         prop_id,
                              const GValue *value,
                              GParamSpec   *pspec);
static gboolean evaluate     (GeglOperation   *operation,
                              const gchar  *output_prop);
static void     associate    (GeglOperation *operation);


G_DEFINE_TYPE (OpSink, op_sink, GEGL_TYPE_OPERATION)


static void
op_sink_class_init (OpSinkClass * klass)
{
  GObjectClass    *object_class = G_OBJECT_CLASS (klass);
  GeglOperationClass *operation_class = GEGL_OPERATION_CLASS (klass);

  object_class->set_property = set_property;
  object_class->get_property = get_property;
  object_class->finalize     = finalize;

  operation_class->evaluate = evaluate;
  operation_class->associate = associate;

  g_object_class_install_property (object_class, PROP_INPUT,
                                   g_param_spec_object ("input",
                                                        "Input",
                                                        "A sub canvas to process",
                                                        CANVAS_TYPE_RECT,
                                                        G_PARAM_CONSTRUCT |
                                                        G_PARAM_READWRITE |
                                                        GEGL_PAD_INPUT));
  g_object_class_install_property (object_class, PROP_PROCESS,
                                   g_param_spec_boolean ("process",
                                                         "Process",
                                                         "A dummy output pad, apply this pad to trigger processing",
                                                         FALSE,
                                                        G_PARAM_READABLE |
                                                        GEGL_PAD_OUTPUT));
}

static void
op_sink_init (OpSink *self)
{
  self->input   = NULL;
  self->process = FALSE;
}

static void
associate (GeglOperation *self)
{
  GeglOperation  *operation    = GEGL_OPERATION (self);
  GObjectClass   *object_class = G_OBJECT_GET_CLASS (self);

  gegl_operation_create_pad (operation,
                             g_object_class_find_property (object_class,
                                                           "input"));
  gegl_operation_create_pad (operation,
                             g_object_class_find_property (object_class,
                                                           "process"));
}

static void
finalize (GObject *object)
{
  OpSink *self = OP_SINK (object);

  self = NULL;
  G_OBJECT_CLASS (op_sink_parent_class)->finalize (object);
}

static void
get_property (GObject    *object,
              guint       prop_id,
              GValue     *value,
              GParamSpec *pspec)
{
  OpSink *self = OP_SINK (object);

  switch (prop_id)
  {
    case PROP_INPUT:
      g_value_set_object (value, self->input);
      break;
    case PROP_PROCESS:
      g_value_set_boolean (value, self->process);
      break;
    default:
      break;
  }
}

static void
set_property (GObject      *object,
              guint         prop_id,
              const GValue *value,
              GParamSpec   *pspec)
{
  OpSink *self = OP_SINK (object);

  switch (prop_id)
  {
    case PROP_INPUT:
      self->input = g_value_get_object(value);
      break;
    default:
      break;
  }
}

static gboolean
evaluate (GeglOperation *operation,
          const gchar   *output_prop)
{
  OpSink      *op_sink = OP_SINK (operation);
  OpSinkClass *klass   = OP_SINK_GET_CLASS (operation);
  gboolean success = FALSE;

  if (op_sink->input != NULL)
    {
      success = klass->evaluate (operation, output_prop);
      g_object_unref (op_sink->input);
    }
  else
    {
      g_warning ("%s received NULL input", G_OBJECT_TYPE_NAME (operation));
    }

  return success;
}
