#ifdef MAGIC_SELF
 
magic_int    (x,        0,    G_MAXINT,  0)
magic_int    (y,        0,    G_MAXINT,  0)
magic_int    (width,    0,    G_MAXINT,  0)
magic_int    (height,   0,    G_MAXINT,  0)
magic_int    (radius,   0,    G_MAXINT,  20)
magic_int    (opacity,  0,    65536,     0)
magic_int    (hardness, 0,    65536,     0)
magic_int    (red,      0,    65536,     65536)
magic_int    (green,    0,    65536,     65536)
magic_int    (blue,     0,    65536,     0)

#else

#define MAGIC_SELF                 "op_stamp.c"
#define MAGIC_SUPER_CLASS_FILTER
#define MAGIC_TypeName             OpStamp
#define MAGIC_TypeNameClass        OpStampClass
#define MAGIC_type_name            op_stamp
#define MAGIC_type_name_init       op_stamp_init
#define MAGIC_type_name_class_init op_stamp_class_init
#include "magic.h"

static void canvas_stamp (Canvas *canvas, 
                          gint    x,
                          gint    y,
                          gint    radius,
                          gint    opacity,
                          gint    hardness,
     	                  gint    r,
                          gint    g,
                          gint    b);

/* Actual image processing code
 ************************************************************************/
static gboolean
evaluate (GeglOperation *operation,
          const gchar   *output_prop)
{
  OpFilter *filter = OP_FILTER(operation);
  OpStamp  *self   = (OpStamp*)(filter);

  CanvasRect *input = filter->input;

  if(strcmp("output", output_prop))
    return FALSE;

  if (filter->in_place)
    {

      canvas_stamp (input->canvas,
                    input->x + self->x,
                    input->y + self->y,
                    self->radius,
                    self->opacity,
                    self->hardness,
                    self->red,
                    self->green,
                    self->blue);

      /* Just throw the old one away */
      if(filter->output)
        g_object_unref(filter->output);

      filter->output = input;
      g_object_ref (filter->output);
    }
  else
    {
       g_warning ("stamp only works in-place for now");
       return FALSE;
    }
  return  TRUE;
}

static void tile_stamp (Tile *tile,
                        gint x,
                        gint y,
                        gint radius,
                        gint opacity,
                        gint hardness,
	                gint r,
	                gint g,
	                gint b)
{
   guchar *data = tile_get_data (tile);
   gint tile_width = tile_get_width (tile);
   gint tile_height = tile_get_height (tile);
   guint radius_squared = (radius*radius);
   guint inner_radius_squared = (radius*hardness/65536);
   gint  soft_range;
   gint i,j;
  
   gint start_y, end_y;
   gint start_x, end_x;
   
   guint *hrowbuf;
/*   guint *vrowbuf;*/

   if (radius == 0)
     return;
   
   start_y = MAX (0, (y - radius) / SPP);
   end_y = MIN (tile_height - 1, (y + radius + (SPP - 1)) / SPP);
   start_x = MAX (0, (x - radius) / SPP);
   end_x = MIN (tile_width - 1, (x + radius + (SPP - 1)) / SPP);
   
   inner_radius_squared = (inner_radius_squared * inner_radius_squared);
   soft_range = radius_squared - inner_radius_squared;

   r = r * 255/65536;
   g = g * 255/65536;
   b = b * 255/65536;
   
   hrowbuf = g_malloc (sizeof (guint) * (end_x - start_x + 1));
/*   vrowbuf = g_malloc (sizeof (guint) * (end_y - start_y + 1));*/
   
   for (i= start_x; i <= end_x; i++)
     hrowbuf[i-start_x] = ((i*SPP)-x)*((i*SPP)-x);
  
   /* revision increase before changing contents */
   /* this could also be interpreted as a request to turn the tile data
    * writable, and thus be the entrypoint for COW
    */
   tile_prepare_write (tile);

   /* TODO : As the brush shape is circular, almost 25% of pixels could be
    *        skipped by having each horizontal row start&end at&to
    *        the brush outline for that given row. */
   /* TODO : Remove conditionals from inside loops. */
   for (j = start_y; j <= end_y; j++)
     {
        guchar *dst = data;
	guint jcalc = ((j*SPP)-y)*((j*SPP)-y);
        dst += (j) * tile_width * 4;
        dst += (start_x) * 4;

        for (i = 0; i <= (end_x - start_x); i++)
          {
            guint o = (hrowbuf [i] + jcalc);
	    
	    if (o < inner_radius_squared)
                o = opacity;
            else if (o < radius_squared)
	      {
                o = (256 - (256 * (o-inner_radius_squared) / (soft_range))) * opacity / 256;
                
                if (o>65536)
                  o=65536;
	      }
	     
            if ((hrowbuf [i] + jcalc) < radius_squared)
              {
                guint red   = dst[0];
                guint green = dst[1];
                guint blue  = dst[2];
             

                red   = (red   * (65536-o) + r * o) / 65536;
                green = (green * (65536-o) + g * o) / 65536;
                blue  = (blue  * (65536-o) + b * o) / 65536;
                 
                if (red>255)
                  red=255;
                if (green>255)
                  green=255;
                if (blue>255)
                  blue=255;

                dst[0] = red;
                dst[1] = green;
                dst[2] = blue;
              }

            dst+=4;
          }
     }
  g_free (hrowbuf);
/*  g_free (vrowbuf);*/
}

/*  might stamp multiple tiles, as long as radius is smaller than the tile size
 *  a maximum of 4 tiles should be affected.
 */
static void canvas_stamp (Canvas *canvas, 
                          gint x,
                          gint y,
                          gint radius,
                          gint opacity,
                          gint hardness,
      	                  gint r,
	                  gint g,
	                  gint b)
{
  Tile *tile;

  gint tile_width = canvas_get_tile_width (canvas);
  gint tile_height = canvas_get_tile_height (canvas);
  
  gint center_tile_x = tile_tile (x / SPP);
  gint center_tile_y = tile_tile (y / SPP);

  if ((radius == 0) || (radius > HARD_MAX_RADIUS))
    return;

  tile_width *= SPP;
  tile_height *= SPP;

    {
      gint tx,ty;
      gint tilex, tiley;

      /*FIXME: depends on SPP and is nasty */
      tx = x % (tile_width);
      ty = y % (tile_height);

      if (x<0)
        tx += (tile_width);
      if (y<0)
        ty += (tile_height);
      
      for (tiley = tile_tile ((y - radius) / SPP); tiley <= tile_tile ((y + radius + (SPP - 1)) / SPP); tiley++)
        for (tilex = tile_tile ((x - radius) / SPP); tilex <= tile_tile ((x + radius + (SPP - 1)) / SPP); tilex++)
	  {
            tile = canvas_get_tile (canvas, tilex & 0xffffff, tiley & 0xffffff);
            tile_stamp (tile, tx + (tile_width * (center_tile_x - tilex)), ty + (tile_height * (center_tile_y - tiley)), radius, opacity, hardness, r,g,b);
	  }
    }
}




#endif
