#ifdef MAGIC_SELF
  
magic_int    (x,        0,        G_MAXINT,  0)
magic_int    (y,        0,        G_MAXINT,  0)
magic_int    (width,    0,        G_MAXINT,  0)
magic_int    (height,   0,        G_MAXINT,  0)
magic_int    (radius,   0,        G_MAXINT,  20)
magic_int    (opacity,  0,        65536,     0)
magic_int    (hardness, 0,        65536,     0)
magic_int    (offset_x, G_MININT, G_MAXINT,  0)
magic_int    (offset_y, G_MININT, G_MAXINT,  0)

#else

#define MAGIC_SELF                 "op_clone_stamp.c"
#define MAGIC_SUPER_CLASS_FILTER
#define MAGIC_TypeName             OpCloneStamp
#define MAGIC_TypeNameClass        OpCloneStampClass
#define MAGIC_type_name            op_clone_stamp
#define MAGIC_type_name_init       op_clone_stamp_init
#define MAGIC_type_name_class_init op_clone_stamp_class_init
#include "magic.h"

/*  might stamp multiple tiles, as long as radius is smaller than the tile size
 *  a maximum of 4 tiles should be affected.
 */
static void canvas_clone_stamp (Canvas *canvas, 
                                gint    x,
                                gint    y,
                                gint    radius,
                                gint    opacity,
                                gint    hardness,
			        gint    offset_x,
			        gint    offset_y);
  
/* Actual image processing code
 ************************************************************************/
static gboolean
evaluate (GeglOperation *operation,
          const gchar   *output_prop)
{
  OpFilter      *filter = OP_FILTER(operation);
  OpCloneStamp  *self   = (OpCloneStamp*)(filter);

  CanvasRect *input = filter->input;

  if(strcmp("output", output_prop))
    return FALSE;

  if (filter->in_place)
    {
      canvas_clone_stamp (input->canvas,
                          (input->x * SPP) + self->x,
                          (input->y * SPP) + self->y,
                          self->radius, self->opacity, self->hardness,
                          self->offset_x, self->offset_y);

      /* Just throw the old one away */
      if(filter->output)
        g_object_unref(filter->output);

      filter->output = input;
      g_object_ref (filter->output);
    }
  else
    {
       g_warning ("stamp only works in-place for now");
       return FALSE;
    }
  return  TRUE;
}

static void tile_clone_stamp (Tile *tile,
                        gint x,
                        gint y,
                        gint radius,
                        gint opacity,
                        gint hardness,
			gint dst_offset_x,
			gint dst_offset_y,
			Tile *src_tile,
			gint src_offset_x,
			gint src_offset_y,
		        gint copy_area_width,
			gint copy_area_height)
{
   guchar *data = tile_get_data (tile);
   guchar *src_data = tile_get_data (src_tile);
   gint tile_width = tile_get_width (tile);
   guint radius_squared = (radius*radius);
   guint inner_radius_squared = (radius*hardness/65536);
   gint  soft_range;
   gint i,j;
  
   gint start_y, end_y;
   gint start_x, end_x;
   
   guint *hrowbuf;
/*   guint *vrowbuf;*/

#if SPP
   gint k,l;
   radius /= SPP;
   k = x % SPP;
   l = y % SPP;
   x /= SPP;
   y /= SPP;
   src_offset_x /= SPP;
   src_offset_y /= SPP;
   dst_offset_x /= SPP;
   dst_offset_y /= SPP;
   copy_area_width /= SPP;
   copy_area_height /= SPP;
#endif
   /* radius not needed here as it's calculated in earlier. */
   start_x = dst_offset_x;
   end_x = dst_offset_x + copy_area_width - 1;
   start_y = dst_offset_y;
   end_y = dst_offset_y + copy_area_height - 1;
   
   inner_radius_squared = (inner_radius_squared * inner_radius_squared);
   soft_range = radius_squared - inner_radius_squared;

   hrowbuf = g_malloc (sizeof (guint) * (end_x - start_x + 1));
/*   vrowbuf = g_malloc (sizeof (guint) * (end_y - start_y + 1));*/
   
   for (i= start_x; i <= end_x; i++)
     hrowbuf[i-start_x] = (((i-x)*SPP)-k)*(((i-x)*SPP)-k);
   
   tile_prepare_write (tile);
   
   for (j = 0; j <= (end_y - start_y); j++)
     {
        guchar *dst = data;
	guchar *src = src_data;
	guint jcalc = (((j+start_y-y)*SPP)-l)*(((j+start_y-y)*SPP)-l);
        dst += (j + dst_offset_y) * tile_width * 4;
        dst += (start_x) * 4;
	src += (j + src_offset_y) * tile_width * 4;
        src += (src_offset_x) * 4;

        for (i = 0; i <= (end_x - start_x); i++)
          {
            guint o = (hrowbuf [i] + jcalc);
	    
	    if (o < inner_radius_squared)
                o = opacity;
            else if (o < radius_squared)
	      {
             
                o = (256 - (256 * (o-inner_radius_squared) / (soft_range))) * opacity / 256;
                
                if (o>65536)
                  o=65536;
		    
	      }
	     
            if ((hrowbuf [i] + jcalc) < radius_squared)
              {
                guint red   = dst[0];
                guint green = dst[1];
                guint blue  = dst[2];

		guint src_red = src[0];
		guint src_green = src[1];
		guint src_blue = src[2];

                red   = (red   * (65536-o) + src_red * o) / 65536;
                green = (green * (65536-o) + src_green * o) / 65536;
                blue  = (blue  * (65536-o) + src_blue * o) / 65536;
                 
                if (red>255)
                  red=255;
                if (green>255)
                  green=255;
                if (blue>255)
                  blue=255;

                dst[0] = red;
                dst[1] = green;
                dst[2] = blue;
              }

	    dst+=4;
	    src+=4;
          }
     }
  g_free (hrowbuf);
/*  g_free (vrowbuf);*/
}


/*  might stamp multiple tiles, as long as radius is smaller than the tile size
 *  a maximum of 4 tiles should be affected.
 */
static void canvas_clone_stamp (Canvas *canvas, 
                                gint x,
                                gint y,
                                gint radius,
                                gint opacity,
                                gint hardness,
                                gint offset_x,
                                gint offset_y)
{
  gint tile_width = canvas_get_tile_width (canvas);
  gint tile_height = canvas_get_tile_height (canvas);

  radius *= SPP;
  tile_width *= SPP;
  tile_height *= SPP;
  offset_x *= SPP;
  offset_y *= SPP;

  gint dst_tile_x;
  gint dst_tile_y;
  
  /* Get affected dst pixels */
  gint dst_start_x = (x - radius);
  gint dst_end_x = (x + radius);
  gint dst_start_y = (y - radius);
  gint dst_end_y = (y + radius);

  for (dst_tile_y = tile_tile (dst_start_y / SPP); dst_tile_y <= tile_tile (dst_end_y / SPP); dst_tile_y++)
    for (dst_tile_x = tile_tile (dst_start_x / SPP); dst_tile_x <= tile_tile (dst_end_x / SPP); dst_tile_x++)
      {
	Tile *dst_tile = canvas_get_tile (canvas, dst_tile_x & 0xffffff, dst_tile_y & 0xffffff);
	
	/* Get all needed src pixels for this dst_tile */
        gint src_start_x = MAX ((dst_tile_x * tile_width + offset_x), (x - radius) + offset_x);
        gint src_end_x = MIN (((dst_tile_x + 1) * tile_width - SPP + offset_x), (x + radius) + offset_x);

        gint src_start_y = MAX ((dst_tile_y * tile_height + offset_y), (y - radius) + offset_y);
        gint src_end_y = MIN (((dst_tile_y + 1) * tile_height - SPP + offset_y), src_end_y = (y + radius) + offset_y);

	gint src_tile_x;
	gint src_tile_y;
	
	/* If this is true, then there's nothing to draw here, move along. */
	if ((src_start_x > src_end_x) || (src_start_y > src_end_y))
	  continue;

	/* For every source tile that will be used in the dest tile... */
        for (src_tile_y = tile_tile (src_start_y / SPP); src_tile_y <= tile_tile (src_end_y / SPP); src_tile_y++)
          for (src_tile_x = tile_tile (src_start_x / SPP); src_tile_x <= tile_tile (src_end_x / SPP); src_tile_x++)
	    {
	      Tile *src_tile = canvas_get_tile (canvas, src_tile_x, src_tile_y);
	      
              g_object_ref (G_OBJECT (src_tile));
	      
	      /* Calculate common area of the current dst and src tiles so that
	         we can keep tile_clone_stamp work on single input/output tiles pair. */
	      gint src_tile_clone_x = MAX (0, src_start_x - (src_tile_x * tile_width));
	      gint src_tile_clone_y = MAX (0, src_start_y - (src_tile_y * tile_height));

	      gint dst_tile_clone_x = MAX (0, (dst_start_x - (dst_tile_x * tile_width)) + MAX (0, (src_tile_x * tile_width - offset_x) - dst_start_x));
	      gint dst_tile_clone_y = MAX (0, (dst_start_y - (dst_tile_y * tile_height)) + MAX (0, (src_tile_y * tile_height - offset_y) - dst_start_y));
	      
	      gint copy_area_width = MIN (dst_end_x - dst_start_x + SPP,
		                     MIN (MAX (SPP, dst_end_x - (dst_tile_clone_x + dst_tile_x * tile_width) + SPP),
				     MIN (tile_width - dst_tile_clone_x,
				     MIN (src_end_x - src_start_x + SPP,
				     MIN (MAX (SPP, src_end_x - (src_tile_clone_x + src_tile_x * tile_width) + SPP),
				     MIN (tile_width - src_tile_clone_x,
					  tile_width
					 ))))));
	      gint copy_area_height = MIN (dst_end_y - dst_start_y + SPP,
		                      MIN (MAX (SPP, dst_end_y - (dst_tile_clone_y + dst_tile_y * tile_height) + SPP),
				      MIN (tile_height - dst_tile_clone_y,
				      MIN (src_end_y - src_start_y + SPP,
				      MIN (MAX (SPP, src_end_y - (src_tile_clone_y + src_tile_y * tile_height) + SPP),
				      MIN (tile_height - src_tile_clone_y,
					   tile_height
					  ))))));

              tile_clone_stamp (dst_tile, 
			        x - (tile_width * dst_tile_x),
			        y - (tile_height * dst_tile_y),
			        radius, opacity, hardness,
				dst_tile_clone_x,
				dst_tile_clone_y,
			        src_tile,
			        src_tile_clone_x,
				src_tile_clone_y,
				copy_area_width,
				copy_area_height);

	      g_object_unref (G_OBJECT (src_tile));
	    }
      }
}


#endif
