#ifdef MAGIC_SELF

magic_int    (x,        G_MININT, G_MAXINT,  0)
magic_int    (y,        G_MININT, G_MAXINT,  0)
magic_int    (width,    0,        G_MAXINT,  0)
magic_int    (height,   0,        G_MAXINT,  0)
magic_int    (red,      0,        G_MAXINT,  65536)
magic_int    (green,    0,        G_MAXINT,  65536)
magic_int    (blue,     0,        G_MAXINT,  0)
magic_int    (alpha,    0,        G_MAXINT,  65536)

#else

#define MAGIC_SELF "op_fill_rect.c"
#define MAGIC_SUPER_CLASS_FILTER
#define MAGIC_TypeName             OpFillRect
#define MAGIC_TypeNameClass        OpFillRectClass
#define MAGIC_type_name            op_fill_rect
#define MAGIC_type_name_init       op_fill_rect_init
#define MAGIC_type_name_class_init op_fill_rect_class_init
#include "magic.h"

/* Actual image processing code
 ************************************************************************/


static void
canvas_fill_rect (Canvas *canvas,
                  gint    x,
                  gint    y,
                  gint    width,
                  gint    height,
                  gint    red,
                  gint    green,
                  gint    blue,
                  gint    alpha);

static gboolean
evaluate (GeglOperation *operation,
          const gchar   *output_prop)
{
  OpFilter   *filter = OP_FILTER(operation);
  OpFillRect *self   = (OpFillRect*)(filter);

  CanvasRect *input = filter->input;

  if(strcmp("output", output_prop))
    return FALSE;

  if (filter->in_place)
    {
      canvas_fill_rect (input->canvas,
                        (input->x + self->x),
                        (input->y + self->y),
                        self->width,
                        self->height,
                        self->red,
                        self->green,
                        self->blue,
                        self->alpha);

      /* Just throw the old one away */
      if(filter->output)
        g_object_unref(filter->output);

      filter->output = input;
      g_object_ref (filter->output);
    }
  else
    {
       g_warning ("fill_rect only works in-place for now");
       return FALSE;
    }

  return  TRUE;
}

void
canvas_fill_rect (Canvas *canvas,
                  gint    x,
                  gint    y,
                  gint    width,
                  gint    height,
                  gint    red,
                  gint    green,
                  gint    blue,
                  gint    alpha)
{
  guchar r = red   * 255 / 65536;
  guchar g = green * 255 / 65536;
  guchar b = blue  * 255 / 65536;
  guchar a = alpha * 255 / 65536;

  guchar *buf = g_malloc (4 * width * height);
  if (buf)
    {
      CanvasRect *rect = canvas_rect_new (canvas, x, y, width, height);
      gint i;
      for (i=0;i<width*height;i++)
        {
          buf[i*4 + 0] = r;
          buf[i*4 + 1] = g;
          buf[i*4 + 2] = b;
          buf[i*4 + 3] = a;
        }
      canvas_rect_import_buf (rect, buf);
      g_object_unref (rect);
      g_free (buf);
    }
}


#endif
